/*
 * Shapes.fx
 *
 */

package apidemo3;

/**
 * @author Jeff Friesen
 */

import java.lang.Math;
import java.lang.System;

import javafx.animation.Interpolator;
import javafx.animation.KeyFrame;
import javafx.animation.Timeline;

import javafx.application.Frame;
import javafx.application.Stage;

import javafx.scene.geometry.Arc;
import javafx.scene.geometry.ArcType;
import javafx.scene.geometry.Circle;
import javafx.scene.geometry.CubicCurve;
import javafx.scene.geometry.Ellipse;
import javafx.scene.geometry.Line;
import javafx.scene.geometry.QuadCurve;
import javafx.scene.geometry.Rectangle;
import javafx.scene.geometry.Shape;

import javafx.scene.paint.Color;
import javafx.scene.paint.LinearGradient;
import javafx.scene.paint.Stop;

class Model
{
    attribute width: Integer;
    attribute height: Integer;
    attribute shape: Shape;
    
    private attribute opacity: Number;

    private function rnd (limit: Integer): Integer
    {
        Math.random ()*limit as Integer
    }
    
    private function arc (): Shape
    {
        Arc
        {
            centerX: rnd (width) 
            centerY: rnd (height)
            radiusX: rnd (Math.min (width/2, height/2))
            radiusY: rnd (Math.min (width/2, height/2))
            startAngle: rnd (180)
            length: rnd (180)
            type: ArcType.ROUND
            fill: Color.rgb (rnd (256), rnd (256), rnd (256))
            opacity: bind opacity       
        }
    }

    private function circle (): Shape
    {
        Circle
        {
            centerX: rnd (width) 
            centerY: rnd (height)
            radius: rnd (Math.min (width/2, height/2))
            fill: Color.rgb (rnd (256), rnd (256), rnd (256))
            opacity: bind opacity
        }
    }

    private function cubicCurve (): Shape
    {
        CubicCurve
        {
            startX: rnd (width) 
            startY: rnd (height)
            controlX1: rnd (width) 
            controlY1: rnd (width) 
            controlX2: rnd (width) 
            controlY2: rnd (width) 
            endX: rnd (width/2)
            endY: rnd (height/2)
            fill: Color.rgb (rnd (256), rnd (256), rnd (256))
            opacity: bind opacity
        }
    }

    private function ellipse (): Shape
    {
        Ellipse
        {
            centerX: rnd (width) 
            centerY: rnd (height)
            radiusX: rnd (Math.min (width/2, height/2))
            radiusY: rnd (Math.min (width/2, height/2))
            fill: Color.rgb (rnd (256), rnd (256), rnd (256))
            opacity: bind opacity
        }
    }

    private function line (): Shape
    {
        Line
        {
            startX: rnd (width)
            startY: rnd (height)
            endX: rnd (width)
            endY: rnd (height)
            stroke: Color.rgb (rnd (256), rnd (256), rnd (256))
            opacity: bind opacity
        }
    }

    private function quadCurve (): Shape
    {
        QuadCurve
        {
            startX: rnd (width) 
            startY: rnd (height)
            endX: rnd (width)
            endY: rnd (height)
            controlX: rnd (width)
            controlY: rnd (height)
            fill: Color.rgb (rnd (256), rnd (256), rnd (256))
            opacity: bind opacity
        }
    }

    function rectangle (): Shape
    {
        Rectangle
        {
            x: rnd (width) 
            y: rnd (height)
            width: rnd (width)
            height: rnd (height)
            arcWidth: rnd (10)
            arcHeight: rnd (10)
            fill: Color.rgb (rnd (256), rnd (256), rnd (256))
            opacity: bind opacity
        }
    }

    private attribute shapes = [arc, circle, cubicCurve, ellipse, line, 
                                quadCurve, rectangle];
                              
    private attribute timeline = Timeline 
    {
        repeatCount: Timeline.INDEFINITE
        keyFrames: for (i in [1..sizeof shapes])
        {
            KeyFrame 
            {
                time: 3s*indexof i
                action: function ()
                {
                    shape = shapes [rnd (sizeof shapes)] ()
                }

                timelines: 
                [
                    Timeline
                    {
                        autoReverse: true
                        repeatCount: 2
                                 
                        var begin = at (0s)
                        {
                            opacity => 0.0
                        }

                        var end = at (1s)
                        {
                            opacity => 1.0 tween Interpolator.LINEAR
                        }
               
                        keyFrames: [begin, end]
                    }
                ]
            }
        }
    }
   
    attribute animate: Boolean on replace o=n
    {
        if (n == true)
            timeline.start ()
        else
            timeline.stop ()
    }
}

Frame
{
    var model = Model
    {
        width: 300
        height: 300
        animate: true
    }
    
    title: "Shapes Tour"
    
    width: bind model.width with inverse
    height: bind model.height with inverse
    
    stage: Stage
    {
        fill: LinearGradient
        {
            startX: 0.0
            startY: 0.0
            endX: 0.0
            endY: 1.0
            stops: 
            [
                Stop { offset: 0.0 color: Color.YELLOW },
                Stop { offset: 1.0 color: Color.CYAN }
            ]
        }
                    
        content: bind [model.shape]
    }

    closeAction: function () 
    { 
        model.animate = false;
        System.exit (0)
    }
    
    visible: true
}
